"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.paginationGenerators = exports.paginationGenerator = exports.getPaginationDefaults = exports.buildPaginationQuery = exports.assignPaginationDefaults = exports.PAGINATION_STEP = exports.PAGINATION_DEFAULTS = void 0;
var _defaults = _interopRequireDefault(require("lodash/defaults"));
var _mapValues = _interopRequireDefault(require("lodash/mapValues"));
var _utils = require("../utils");
var _types = require("./types");
const PAGINATION_STEP = exports.PAGINATION_STEP = 20;
const PAGINATION_DEFAULTS = exports.PAGINATION_DEFAULTS = {
  per_page: PAGINATION_STEP,
  page: 1
};
const getPaginationDefaults = () => ({
  currentPage: 0,
  lastFetched: 0,
  total: null,
  collection: []
});

/**
 * Provides updates for a given type and state to handle list loading, successful fetch and
 * failed fetch.
 */
exports.getPaginationDefaults = getPaginationDefaults;
const paginationGenerator = type => function () {
  let state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
  let action = arguments.length > 1 ? arguments[1] : undefined;
  let update = null;
  if (typeof state.currentPage !== 'number') update = getPaginationDefaults();
  switch (action.type) {
    case type:
      {
        const currentPage = (state.currentPage || 0) + 1;
        update = {
          currentPage,
          isFetching: true,
          isFailed: false
        };
        break;
      }
    case (0, _types.resolved)(type):
      {
        update = {
          isFetching: false,
          lastFetched: Date.now()
        };
        if (action.payload && (0, _utils.has)(action.payload, 'total_count')) {
          update.total = action.payload.total_count;
        }
        break;
      }
    case (0, _types.rejected)(type):
      {
        const currentPage = Math.max((state.currentPage || 0) - 1, 0);
        update = {
          currentPage,
          isFetching: false,
          isFailed: true
        };
        break;
      }
  }
  return update;
};

/**
 * Same thing as `paginationGenerator` but supports multiple lists.
 */
exports.paginationGenerator = paginationGenerator;
const paginationGenerators = typesByStateKey => {
  const updaters = (0, _mapValues.default)(typesByStateKey, type => paginationGenerator(type));
  return function () {
    let state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    let action = arguments.length > 1 ? arguments[1] : undefined;
    return Object.keys(updaters).reduce((updates, stateKey) => {
      const update = updaters[stateKey](state[stateKey], action);
      if (!update) return updates;
      return {
        ...updates,
        [stateKey]: update
      };
    }, {});
  };
};
exports.paginationGenerators = paginationGenerators;
const assignPaginationDefaults = state => ({
  ...state,
  ...getPaginationDefaults()
});
exports.assignPaginationDefaults = assignPaginationDefaults;
const buildPaginationQuery = (query, options) => {
  if (options && ((0, _utils.has)(options, 'last') || (0, _utils.has)(options, 'first'))) {
    const {
      last,
      first
    } = options;
    const pagination = {
      per_page: options.per_page || PAGINATION_DEFAULTS.per_page
    };
    if (last) pagination.lower = last;
    if (first) pagination.higher = first;
    return (0, _defaults.default)({}, query, pagination);
  }
  return (0, _defaults.default)({}, query, options, PAGINATION_DEFAULTS);
};
exports.buildPaginationQuery = buildPaginationQuery;