"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RESOLVED = exports.REJECTED = void 0;
Object.defineProperty(exports, "configurePromise", {
  enumerable: true,
  get: function () {
    return _configuration.configurePromise;
  }
});
exports.resolved = exports.rejected = exports.middleware = void 0;
var _configuration = require("./configuration");
const RESOLVED = exports.RESOLVED = 'RESOLVED';
const REJECTED = exports.REJECTED = 'REJECTED';
const resolved = type => `${type}_${RESOLVED}`;
exports.resolved = resolved;
const rejected = type => `${type}_${REJECTED}`;
exports.rejected = rejected;
const middleware = store => next => action => {
  if (!action.promise) return next(action);
  const {
    promise: options,
    ...cleanAction
  } = action;
  const state = store.getState();

  // prevent unnecessary calls
  if (options.shouldExecute && !options.shouldExecute(state)) {
    return Promise.resolve(null);
  }
  next(cleanAction);
  let promise = options.getPromise?.();

  // legacy: options can be a raw promise
  // TODO: remove in next major release
  if (!promise) promise = options;
  return promise.then(payload => {
    next({
      ...cleanAction,
      type: resolved(action.type),
      payload
    });
  }, payload => {
    next({
      ...cleanAction,
      type: rejected(action.type),
      payload
    });
    (0, _configuration.getErrorHandler)()?.(payload, action);
    if (!options.graceful) return Promise.reject(payload);
  });
};
exports.middleware = middleware;